@@ -14,7 +14,6 @@
 //! Make a backup by walking a source directory and copying the contents
 //! into an archive.
 
-use std::convert::TryInto;
 use std::fmt;
 use std::io::prelude::*;
 use std::mem::take;
@@ -26,17 +25,15 @@ use std::time::{Duration, Instant};
 use bytes::BytesMut;
 use derive_more::{Add, AddAssign};
 use itertools::Itertools;
-use tracing::{error, warn};
+use tracing::{trace, warn};
 
 use crate::blockdir::Address;
 use crate::change::Change;
 use crate::counters::Counter;
-use crate::entry::EntryValue;
 use crate::io::read_with_retries;
 use crate::monitor::Monitor;
 use crate::stats::{write_compressed_size, write_count, write_duration, write_size};
 use crate::stitch::IterStitchedIndexHunks;
-use crate::tree::ReadTree;
 use crate::*;
 
 /// Configuration of how to make a backup.
@@ -46,13 +43,16 @@ pub struct BackupOptions<'cb> {
 
     pub max_entries_per_hunk: usize,
 
-    // Call this callback as each entry is successfully stored.
+    /// Call this callback as each entry is successfully stored.
     pub change_callback: Option<ChangeCallback<'cb>>,
 
     pub max_block_size: usize,
 
     /// Combine files smaller than this into a single block.
     pub small_file_cap: u64,
+
+    /// Record the user/group owners on Unix.
+    pub owner: bool,
 }
 
 impl Default for BackupOptions<'_> {
@@ -63,6 +63,7 @@ impl Default for BackupOptions<'_> {
             change_callback: None,
             max_block_size: 20 << 20,
             small_file_cap: 1 << 20,
+            owner: true,
         }
     }
 }
@@ -89,18 +90,22 @@ pub fn backup(
     monitor: Arc<dyn Monitor>,
 ) -> Result<BackupStats> {
     let start = Instant::now();
-    let mut writer = BackupWriter::begin(archive, options)?;
+    let mut writer = BackupWriter::begin(archive, options, monitor.clone())?;
     let mut stats = BackupStats::default();
     let source_tree = LiveTree::open(source_path)?;
 
     let task = monitor.start_task("Backup".to_string());
 
-    let entry_iter = source_tree.iter_entries(Apath::root(), options.exclude.clone())?;
+    let entry_iter =
+        source_tree.iter_entries(Apath::root(), options.exclude.clone(), monitor.clone())?;
     for entry_group in entry_iter.chunks(options.max_entries_per_hunk).into_iter() {
-        for entry in entry_group {
+        for mut entry in entry_group {
+            if !options.owner {
+                entry.owner.clear();
+            }
             match writer.copy_entry(&entry, &source_tree, options, monitor.clone()) {
                 Err(err) => {
-                    error!(?entry, ?err, "Error copying entry to backup");
+                    monitor.error(err);
                     stats.errors += 1;
                     continue;
                 }
@@ -150,12 +155,20 @@ impl BackupWriter {
     /// Create a new BackupWriter.
     ///
     /// This currently makes a new top-level band.
-    pub fn begin(archive: &Archive, options: &BackupOptions) -> Result<Self> {
+    pub fn begin(
+        archive: &Archive,
+        options: &BackupOptions,
+        monitor: Arc<dyn Monitor>,
+    ) -> Result<Self> {
         if gc_lock::GarbageCollectionLock::is_locked(archive)? {
             return Err(Error::GarbageCollectionLockHeld);
         }
-        let basis_index = IterStitchedIndexHunks::new(archive, archive.last_band_id()?)
-            .iter_entries(Apath::root(), Exclude::nothing());
+        let basis_index = if let Some(basis_band_id) = archive.last_band_id()? {
+            IterStitchedIndexHunks::new(archive, basis_band_id, monitor)
+        } else {
+            IterStitchedIndexHunks::empty(archive, monitor)
+        }
+        .iter_entries(Apath::root(), Exclude::nothing());
 
         // Create the new band only after finding the basis band!
         let band = Band::create(archive)?;
@@ -235,24 +248,23 @@ impl BackupWriter {
         monitor.count(Counter::Files, 1);
         let apath = source_entry.apath();
         let result = if let Some(basis_entry) = self.basis_index.advance_to(apath) {
-            if entry_metadata_unchanged(source_entry, &basis_entry) {
-                if basis_entry
-                    .addrs
-                    .iter()
-                    .map(|addr| &addr.hash)
-                    .unique()
-                    .all(|hash| {
-                        self.block_dir
-                            .contains(hash, monitor.clone())
-                            .unwrap_or(false)
-                    })
-                {
+            if content_heuristically_unchanged(source_entry, &basis_entry) {
+                if all_blocks_present(&basis_entry.addrs, &self.block_dir, &monitor) {
                     self.stats.unmodified_files += 1;
-                    let change = Some(EntryChange::unchanged(&basis_entry));
-                    self.index_builder.push_entry(basis_entry);
-                    return Ok(change);
+                    let new_entry = IndexEntry {
+                        addrs: basis_entry.addrs.clone(),
+                        ..IndexEntry::metadata_from(source_entry)
+                    };
+                    let change = if new_entry == basis_entry {
+                        EntryChange::unchanged(&basis_entry)
+                    } else {
+                        trace!(%apath, "Content same, metadata changed");
+                        EntryChange::changed(&basis_entry, source_entry)
+                    };
+                    self.index_builder.push_entry(new_entry);
+                    return Ok(Some(change));
                 } else {
-                    warn!("Some blocks referenced by {apath:?} are missing from the blockdir; file will be stored again");
+                    warn!(%apath, "Some referenced blocks are missing or truncated; file will be stored again");
                     self.stats.modified_files += 1;
                     self.stats.replaced_damaged_blocks += 1;
                     Some(EntryChange::changed(&basis_entry, source_entry))
@@ -311,6 +323,18 @@ impl BackupWriter {
     }
 }
 
+fn all_blocks_present(
+    addresses: &[Address],
+    block_dir: &BlockDir,
+    monitor: &Arc<dyn Monitor>,
+) -> bool {
+    addresses
+        .iter()
+        .map(|addr| &addr.hash)
+        .unique()
+        .all(|hash| block_dir.contains(hash, monitor.clone()).unwrap_or(false))
+}
+
 fn store_file_content(
     apath: &Apath,
     from_file: &mut dyn Read,
@@ -500,12 +524,13 @@ impl FileCombiner {
 /// not changed, without reading the file content.
 ///
 /// Caution: this does not check the symlink target.
-fn entry_metadata_unchanged<E: EntryTrait, O: EntryTrait>(new_entry: &E, basis_entry: &O) -> bool {
+fn content_heuristically_unchanged<E: EntryTrait, O: EntryTrait>(
+    new_entry: &E,
+    basis_entry: &O,
+) -> bool {
     basis_entry.kind() == new_entry.kind()
         && basis_entry.mtime() == new_entry.mtime()
         && basis_entry.size() == new_entry.size()
-        && basis_entry.unix_mode() == new_entry.unix_mode()
-        && basis_entry.owner() == new_entry.owner()
 }
 
 #[derive(Add, AddAssign, Debug, Default, Eq, PartialEq, Clone)]
