@@ -20,26 +20,32 @@ use std::path::{Path, PathBuf};
 use std::sync::{Arc, RwLock};
 use std::time::Instant;
 
+use clap::builder::{styling, Styles};
 use clap::{Parser, Subcommand};
 use conserve::change::Change;
-use conserve::trace_counter::{global_error_count, global_warn_count};
 use rayon::prelude::ParallelIterator;
 use time::UtcOffset;
 #[allow(unused_imports)]
 use tracing::{debug, error, info, trace, warn, Level};
 
-use conserve::backup::BackupOptions;
 use conserve::termui::{enable_tracing, TermUiMonitor, TraceTimeStyle};
-use conserve::ReadTree;
-use conserve::RestoreOptions;
 use conserve::*;
 
 /// Local timezone offset, calculated once at startup, to avoid issues about
 /// looking at the environment once multiple threads are running.
 static LOCAL_OFFSET: RwLock<UtcOffset> = RwLock::new(UtcOffset::UTC);
 
+#[mutants::skip] // only visual effects, not worth testing
+fn clap_styles() -> Styles {
+    styling::Styles::styled()
+        .header(styling::AnsiColor::Green.on_default() | styling::Effects::BOLD)
+        .usage(styling::AnsiColor::Green.on_default() | styling::Effects::BOLD)
+        .literal(styling::AnsiColor::Blue.on_default() | styling::Effects::BOLD)
+        .placeholder(styling::AnsiColor::Cyan.on_default())
+}
+
 #[derive(Debug, Parser)]
-#[command(author, about, version)]
+#[command(author, about, version, styles(clap_styles()))]
 struct Args {
     #[command(subcommand)]
     command: Command,
@@ -114,8 +120,11 @@ enum Command {
 
     /// Compare a stored tree to a source directory.
     Diff {
+        /// Path or URL of an existing archive.
         archive: String,
+        /// Source directory to compare to.
         source: PathBuf,
+        /// Select the version from the archive to compare: by default, the latest.
         #[arg(long, short)]
         backup: Option<BandId>,
         #[arg(long, short)]
@@ -317,10 +326,6 @@ impl Command {
                     )?,
                     ..Default::default()
                 };
-                if *long_listing || *verbose {
-                    // TODO(CON-23): Really Nutmeg should coordinate stdout and stderr...
-                    // todo!("Disable progress bars!");
-                }
                 let stats = backup(
                     &Archive::open(open_transport(archive)?)?,
                     source,
@@ -399,7 +404,7 @@ impl Command {
                     include_unchanged: *include_unchanged,
                 };
                 let mut bw = BufWriter::new(stdout);
-                for change in diff(&st, &lt, &options)? {
+                for change in diff(&st, &lt, &options, monitor.clone())? {
                     if *json {
                         serde_json::to_writer(&mut bw, &change)?;
                     } else {
@@ -443,14 +448,15 @@ impl Command {
                         // TODO: Option for subtree.
                         Box::new(
                             stored_tree_from_opt(archive, &stos.backup)?
-                                .iter_entries(Apath::root(), exclude)?
+                                .iter_entries(Apath::root(), exclude, monitor.clone())?
                                 .map(|it| it.into()),
                         )
                     } else {
-                        Box::new(
-                            LiveTree::open(stos.source.clone().unwrap())?
-                                .iter_entries(Apath::root(), exclude)?,
-                        )
+                        Box::new(LiveTree::open(stos.source.clone().unwrap())?.iter_entries(
+                            Apath::root(),
+                            exclude,
+                            monitor.clone(),
+                        )?)
                     };
                 monitor.clear_progress_bars();
                 if *json {
@@ -471,11 +477,12 @@ impl Command {
                 exclude,
                 exclude_from,
                 only_subtree,
-                no_stats,
                 long_listing,
+                no_stats,
             } => {
                 let band_selection = band_selection_policy_from_opt(backup);
                 let archive = Archive::open(open_transport(archive)?)?;
+                let _ = no_stats; // accepted but ignored; we never currently print stats
                 let options = RestoreOptions {
                     exclude: Exclude::from_patterns_and_files(exclude, exclude_from)?,
                     only_subtree: only_subtree.clone(),
@@ -487,14 +494,8 @@ impl Command {
                         &changes_json.as_deref(),
                     )?,
                 };
-                if *verbose || *long_listing {
-                    // todo!("Disable progress bar");
-                }
-                let stats = restore(&archive, destination, &options, monitor)?;
+                restore(&archive, destination, &options, monitor)?;
                 debug!("Restore complete");
-                if !no_stats {
-                    debug!(%stats);
-                }
             }
             Command::Size {
                 stos,
@@ -523,8 +524,8 @@ impl Command {
                 let options = ValidateOptions {
                     skip_block_hashes: *quick,
                 };
-                Archive::open(open_transport(archive)?)?.validate(&options, monitor)?;
-                if global_error_count() > 0 || global_warn_count() > 0 {
+                Archive::open(open_transport(archive)?)?.validate(&options, monitor.clone())?;
+                if monitor.error_count() != 0 {
                     warn!("Archive has some problems.");
                 } else {
                     info!("Archive is OK.");
@@ -645,18 +646,12 @@ fn main() -> Result<ExitCode> {
             monitor.counters(),
         )?;
     }
-    let error_count = global_error_count();
-    let warn_count = global_warn_count();
     match result {
         Err(err) => {
             error!("{err:#}");
-            debug!(error_count, warn_count,);
             Ok(ExitCode::Failure)
         }
-        Ok(ExitCode::Success) if error_count > 0 || warn_count > 0 => {
-            debug!(error_count, warn_count,);
-            Ok(ExitCode::NonFatalErrors)
-        }
+        Ok(ExitCode::Success) if monitor.error_count() != 0 => Ok(ExitCode::NonFatalErrors),
         Ok(exit_code) => Ok(exit_code),
     }
 }
