@@ -22,7 +22,6 @@
 //! The structure is: archive > blockdir > subdir > file.
 
 use std::collections::{HashMap, HashSet};
-use std::convert::TryInto;
 use std::sync::atomic::AtomicUsize;
 use std::sync::atomic::Ordering::Relaxed;
 use std::sync::{Arc, RwLock};
@@ -31,18 +30,16 @@ use bytes::Bytes;
 use lru::LruCache;
 use rayon::prelude::*;
 use serde::{Deserialize, Serialize};
-use tracing::{debug, error, warn};
+use tracing::{debug, warn};
 use tracing::{instrument, trace};
 
-use crate::backup::BackupStats;
-use crate::blockhash::BlockHash;
 use crate::compress::snappy::{Compressor, Decompressor};
 use crate::counters::Counter;
 use crate::monitor::Monitor;
-use crate::transport::{ListDir, Transport};
+use crate::transport::ListDir;
 use crate::*;
 
-const BLOCKDIR_FILE_NAME_LEN: usize = crate::BLAKE_HASH_SIZE_BYTES * 2;
+// const BLOCKDIR_FILE_NAME_LEN: usize = crate::BLAKE_HASH_SIZE_BYTES * 2;
 
 /// Take this many characters from the block hash to form the subdirectory name.
 const SUBDIR_NAME_CHARS: usize = 3;
@@ -90,7 +87,7 @@ pub fn block_relpath(hash: &BlockHash) -> String {
 impl BlockDir {
     pub fn open(transport: Arc<dyn Transport>) -> BlockDir {
         /// Cache this many blocks in memory.
-        // TODO: Change to a cache that tracks the size of strored blocks?
+        // TODO: Change to a cache that tracks the size of stored blocks?
         // As a safe conservative value, 100 blocks of 20MB each would be 2GB.
         const BLOCK_CACHE_SIZE: usize = 100;
 
@@ -193,9 +190,10 @@ impl BlockDir {
         let end = start + len;
         let actual_len = bytes.len();
         if end > actual_len {
-            return Err(Error::AddressTooLong {
-                address: address.to_owned(),
+            return Err(Error::BlockTooShort {
+                hash: address.hash.clone(),
                 actual_len,
+                referenced_len: len,
             });
         }
         Ok(bytes.slice(start..end))
@@ -219,7 +217,6 @@ impl BlockDir {
         let decompressed_bytes = decompressor.decompress(&compressed_bytes)?;
         let actual_hash = BlockHash::hash_bytes(&decompressed_bytes);
         if actual_hash != *hash {
-            error!(%hash, %actual_hash, %block_relpath, "Block file has wrong hash");
             return Err(Error::BlockCorrupt { hash: hash.clone() });
         }
         self.cache
@@ -283,15 +280,17 @@ impl BlockDir {
                 task.increment(1);
                 r
             })
-            .filter_map(|iter_or| {
-                if let Err(ref err) = iter_or {
-                    error!(%err, "Error listing block subdirectory");
+            .filter_map(move |iter_or| match iter_or {
+                Err(source) => {
+                    monitor.error(Error::ListBlocks { source });
+                    None
                 }
-                iter_or.ok()
+                Ok(ListDir { files, .. }) => Some(files),
             })
-            .flat_map(|ListDir { files, .. }| files)
-            .filter(|name| name.len() == BLOCKDIR_FILE_NAME_LEN && !name.starts_with(TMP_PREFIX))
-            .filter_map(|name| name.parse().ok()))
+            .flatten()
+            .filter_map(|name| // drop any invalid names, including temp files
+                // TODO: Report errors on bad names?
+                name.parse().ok()))
     }
 
     /// Check format invariants of the BlockDir.
@@ -319,7 +318,7 @@ impl BlockDir {
                         Some((hash, bytes.len()))
                     }
                     Err(err) => {
-                        error!(%err, %hash, "Error reading block content");
+                        monitor.error(err);
                         None
                     }
                 },
@@ -339,13 +338,15 @@ pub struct BlockDirStats {
 
 #[cfg(test)]
 mod test {
-    use std::fs::OpenOptions;
+    use std::fs::{create_dir, write, OpenOptions};
 
-    use crate::monitor::collect::CollectMonitor;
+    use tempfile::TempDir;
+
+    use crate::monitor::test::TestMonitor;
     use crate::transport::open_local_transport;
 
     use super::*;
-    use tempfile::TempDir;
+
     #[test]
     fn empty_block_file_counts_as_not_present() {
         // Due to an interruption or system crash we might end up with a block
@@ -354,7 +355,7 @@ mod test {
         let tempdir = TempDir::new().unwrap();
         let blockdir = BlockDir::open(open_local_transport(tempdir.path()).unwrap());
         let mut stats = BackupStats::default();
-        let monitor = CollectMonitor::arc();
+        let monitor = TestMonitor::arc();
         let hash = blockdir
             .store_or_deduplicate(Bytes::from("stuff"), &mut stats, monitor.clone())
             .unwrap();
@@ -367,7 +368,7 @@ mod test {
 
         // Open again to get a fresh cache
         let blockdir = BlockDir::open(open_local_transport(tempdir.path()).unwrap());
-        let monitor = CollectMonitor::arc();
+        let monitor = TestMonitor::arc();
         OpenOptions::new()
             .write(true)
             .truncate(true)
@@ -379,6 +380,25 @@ mod test {
         assert_eq!(monitor.get_counter(Counter::BlockExistenceCacheMiss), 1);
     }
 
+    #[test]
+    fn temp_files_are_not_returned_as_blocks() {
+        let tempdir = TempDir::new().unwrap();
+        let blockdir = BlockDir::open(open_local_transport(tempdir.path()).unwrap());
+        let monitor = TestMonitor::arc();
+        let subdir = tempdir.path().join(subdir_relpath("123"));
+        create_dir(&subdir).unwrap();
+        write(
+            subdir.join(format!("{}{}", TMP_PREFIX, "123123123")),
+            b"123",
+        )
+        .unwrap();
+        let blocks = blockdir
+            .blocks(monitor.clone())
+            .unwrap()
+            .collect::<Vec<_>>();
+        assert_eq!(blocks, []);
+    }
+
     #[test]
     fn cache_hit() {
         let tempdir = TempDir::new().unwrap();
@@ -386,16 +406,16 @@ mod test {
         let mut stats = BackupStats::default();
         let content = Bytes::from("stuff");
         let hash = blockdir
-            .store_or_deduplicate(content.clone(), &mut stats, CollectMonitor::arc())
+            .store_or_deduplicate(content.clone(), &mut stats, TestMonitor::arc())
             .unwrap();
         assert_eq!(blockdir.stats.cache_hit.load(Relaxed), 0);
 
-        let monitor = CollectMonitor::arc();
+        let monitor = TestMonitor::arc();
         assert!(blockdir.contains(&hash, monitor.clone()).unwrap());
         assert_eq!(blockdir.stats.cache_hit.load(Relaxed), 1);
         assert_eq!(monitor.get_counter(Counter::BlockExistenceCacheHit), 1);
 
-        let monitor = CollectMonitor::arc();
+        let monitor = TestMonitor::arc();
         let retrieved = blockdir.get_block_content(&hash, monitor.clone()).unwrap();
         assert_eq!(content, retrieved);
         assert_eq!(monitor.get_counter(Counter::BlockContentCacheHit), 1);
@@ -415,13 +435,13 @@ mod test {
         let blockdir = BlockDir::open(open_local_transport(tempdir.path()).unwrap());
         let mut stats = BackupStats::default();
         let content = Bytes::from("stuff");
-        let monitor = CollectMonitor::arc();
+        let monitor = TestMonitor::arc();
         let hash = blockdir
             .store_or_deduplicate(content.clone(), &mut stats, monitor.clone())
             .unwrap();
 
         // reopen
-        let monitor = CollectMonitor::arc();
+        let monitor = TestMonitor::arc();
         let blockdir = BlockDir::open(open_local_transport(tempdir.path()).unwrap());
         assert!(blockdir.contains(&hash, monitor.clone()).unwrap());
         assert_eq!(blockdir.stats.cache_hit.load(Relaxed), 0);
